import pygame
import os
import model.wall
import collections
import model.tank
import model.pit
import model.enemy


# Width x Height
TILE_SIZE = (40, 40)
MAP_DIR = os.path.join(os.path.dirname(__file__), 'maps')


MAP_CHARACTERS = {
        '#': model.wall.Wall,
        'T': model.tank.Tank,
        'P': model.pit.Pit,
        'E': model.enemy.Enemy,
        ' ': None,
        }


class UnknownMapCharacter(Exception):
    
    def __init__(self, char):
        Exception.__init__(self, "'%s' is not a valid Map Character" % char)


class DimensionError(Exception):
    
    """
    Dimension for a map did not match dimensions given
    """

    pass


class Map(object):

    def __init__(self, map_filename=None):
        self.map = []
        self.__map_filename = None
        self.sprites = collections.defaultdict(pygame.sprite.RenderUpdates)
        self.map_filename = map_filename

    def __get_map_filename(self):
        return self.__map_filename

    def __set_map_filename(self, v):
        self.__map_filename = v
        self.__read_map(open(os.path.join(MAP_DIR, v), 'r'))

    map_filename = property(__get_map_filename, __set_map_filename)

    def __read_map(self, stream):
        self.__width, self.__height = stream.readline().split()
        self.__width = int(self.__width)
        self.__height = int(self.__height)
        for line in stream:
            parsed_line = []
            for char in line:
                if char == '\n':
                    break
                if char not in MAP_CHARACTERS:
                    raise UnknownMapCharacter(char)
                if MAP_CHARACTERS[char] is None:
                    parsed_line.append(None)
                    continue
                new_item = MAP_CHARACTERS[char]()
                new_item.rect.top += TILE_SIZE[1] * len(self.map)
                new_item.rect.left += TILE_SIZE[0] * len(parsed_line)
                parsed_line.append(new_item)
                self.sprites[MAP_CHARACTERS[char]].add(new_item)
            if len(parsed_line) != self.__width:
                raise DimensionError("Width did not match on line '%s'" % line)
            self.map.append(parsed_line)
        if len(self.map) != self.__height:
            raise DimensionError("Height did not match")


def __test():
    m = Map('testmap.txt')


if __name__ == '__main__':
    pygame.init()
    screen = pygame.display.set_mode((320, 240))
    __test()


