import pygame
import weakref
from . import _collision_union, images
from .direction import Direction
from copy import copy
import random
import sound


NUM_TIMESTEP = 100
VELOCITY = 1


class Enemy(pygame.sprite.Sprite):

    def __init__(self):
        pygame.sprite.Sprite.__init__(self)
        self.image, self.rect = images.get_image('enemy1')
        self.__walls = lambda: None
        self.timestep = 0
        self.direction = Direction.UP
        self.colliding = False
        self.__animation = 1
        self.__right = True
        self.die_sound = sound.get_sound('man')

    def set_walls(self, walls):
        self.__walls = weakref.ref(walls)

    def update(self):
        oldrect = copy(self.rect)
        if self.timestep >= NUM_TIMESTEP or self.colliding:
            self.timestep = -1
            movement_dirs = set((Direction.UP, Direction.DOWN,
                    Direction.LEFT, Direction.RIGHT)) - set((self.direction,))
            self.direction = random.choice(tuple(movement_dirs))
        if self.direction == Direction.UP:
            self.rect.move_ip(0, -VELOCITY)
        elif self.direction == Direction.RIGHT:
            self.__right = True
            self.rect.move_ip(VELOCITY, 0)
        elif self.direction == Direction.DOWN:
            self.rect.move_ip(0, VELOCITY)
        elif self.direction == Direction.LEFT:
            self.__right = False
            self.rect.move_ip(-VELOCITY, 0)
        topleft, self.colliding = \
                _collision_union.collision(self, self.__walls(), oldrect)
        self.rect.topleft = topleft
        self.timestep += 1
        #animation
        oldcenter = self.rect.center
        if self.__animation == 1:
            self.__animation = 2
            self.image, self.rect = images.get_image('enemy2')
        elif self.__animation == 2:
            self.__animation = 3
            self.image, self.rect = images.get_image('enemy3')
        elif self.__animation == 3:
            self.__animation = 4
            self.image, self.rect = images.get_image('enemy4')
        else:
            self.__animation = 1
            self.image, self.rect = images.get_image('enemy1')
        if self.__right:
            self.image = pygame.transform.flip(self.image, 0, 0)
            self.rect = self.image.get_rect()
        else:
            self.image = pygame.transform.flip(self.image, 1, 0)
            self.rect = self.image.get_rect()
        self.rect.center = oldcenter

