import pygame
from . import images
from . import sound, _collision_union
from direction import Direction
from strength import Strength
from copy import copy
import logging
import weakref

logger = logging.getLogger(__name__)

VELOCITY = 1


class ImpossiblePositioningException(Exception):
    pass


class Tank(pygame.sprite.Sprite):
    def __init__(self):
        pygame.sprite.Sprite.__init__(self)
        self.base, self.rect = images.get_image('tank1')
        self.image, self.rect = images.get_image('tank1')
        self.__turning_right = False
        self.__direction = Direction.UP
        self.__charge = 0
        self.__fire = False
        self.__fire_sound = sound.get_sound('fire')
        self.__die_sound = sound.get_sound('building')
        self.__health = 30
        self.__rotation = 0
        self.__walls = lambda: None
        self.__animation = 1
        self.__no_damage = 0

    def set_walls(self, walls):
        self.__walls = weakref.ref(walls)
    
    def turn_right(self):
        self.__turning_right = True
        #self.fire()
    
    def charge(self):
        self.__turning_right = False
        self.__fire = True

    def clear_charge(self):
        self.__fire = False
        self.__charge = 0
    
    def fire(self):
        self.__fire = False
        self.__fire_sound.play()
    
    def update(self):
        #turning
        if self.__turning_right:
            self.__rotation = (self.__rotation - 90) % 360
            if self.__direction == Direction.UP:
                self.__direction = Direction.RIGHT
            elif self.__direction == Direction.RIGHT:
                self.__direction = Direction.DOWN
            elif self.__direction == Direction.DOWN:
                self.__direction = Direction.LEFT
            else:
                self.__direction = Direction.UP
            self.__turning_right = False
        #moving forward
        else:
            old_rect = copy(self.rect)
            if self.__direction == Direction.UP:
                self.rect.top -= VELOCITY
            elif self.__direction == Direction.RIGHT:
                self.rect.left += VELOCITY
            elif self.__direction == Direction.DOWN:
                self.rect.top += VELOCITY
            else:
                self.rect.left -= VELOCITY
            if self.__walls() is not None:
                topleft, collide = \
                    _collision_union.collision(self, self.__walls(), old_rect)
                if collide:
                    self.rect.topleft = topleft
        #charge
        if self.__fire:
            self.__charge += 1
        #fire
        elif not self.__fire and self.__charge > 0:
            self.__charge = 0
        #animation
        oldcenter = self.rect.center
        if self.__animation == 3:
            self.__animation = 2
            self.base, self.rect = images.get_image('tank2')
        elif self.__animation == 2:
            self.__animation = 1
            self.base, self.rect = images.get_image('tank1')
        else:
            self.__animation = 3
            self.base, self.rect = images.get_image('tank3')
        if self.__no_damage > 0:
            self.__no_damage -= 1
            if self.__no_damage % 6 < 4:
                self.base, self.rect = images.get_image('tank4')
        self.rect = self.image.get_rect()
        self.image = pygame.transform.rotate(self.base, self.__rotation)
        self.rect.center = oldcenter

    def get_direction(self):
        return self.__direction

    def get_position(self):
        return (self.rect.left, self.rect.top)

    def get_health(self):
        return self.__health

    def get_charge(self):
        return self.__charge

    def shot(self, strength):
        """
        Returns True if tank dies
        """
        if self.__no_damage == 0:
            self.__health -= strength
            self.__no_damage = 40
        if self.__health <= 0:
            self.__die_sound
            return True
        else:
            return False

